//---------------------------------------------------------------------------
//
//   MQPlugin.h      Copyright(C) 1999-2000, O.Mizno
//
//    @Metasequoia SDK Rev0.07̎dlɊÂCN[ht@CB
//    vOCJ҂̃t@C̓eύXKv͂ȂB
//
//---------------------------------------------------------------------------

#ifndef MQPluginH
#define MQPluginH


#ifdef MQPLUGIN_EXPORTS
#define MQPLUGIN_API extern "C" __declspec(dllexport) __stdcall
#else
#define MQPLUGIN_API extern "C"
#endif

#define MQPLUGIN_EXPORT extern "C" __declspec(dllexport)


#define MQPLUGIN_TYPE_IMPORT   1
#define MQPLUGIN_TYPE_EXPORT   2
#define MQPLUGIN_TYPE_CREATE   3
#define MQPLUGIN_TYPE_OBJECT   4
#define MQPLUGIN_TYPE_SELECT   5

// MQ_ShowFileDialog
#define MQFILE_TYPE_LEFT  0
#define MQFILE_TYPE_RIGHT 1
#define MQFILE_TYPE_UP    2
#define MQFILE_TYPE_DOWN  3
#define MQFILE_TYPE_FRONT 4
#define MQFILE_TYPE_BACK  5

// MQDoc_ClearSelect
#define MQDOC_CLEARSELECT_VERTEX  1
#define MQDOC_CLEARSELECT_LINE    2
#define MQDOC_CLEARSELECT_FACE    4
#define MQDOC_CLEARSELECT_ALL     7

// MQObj_Freeze
#define MQOBJECT_FREEZE_PATCH    0x00000001
#define MQOBJECT_FREEZE_MIRROR   0x00000002
#define MQOBJECT_FREEZE_ALL      0x7FFFFFFF

// MQObj_GetPatchType, MQObj_SetPatchType
#define MQOBJECT_PATCH_NONE      0
#define MQOBJECT_PATCH_SPLINE1   1
#define MQOBJECT_PATCH_SPLINE2   2


typedef struct MQPoint
{
	float x,y,z;
public:
	// constructor
	inline MQPoint() { }
	inline MQPoint(float nx, float ny, float nz) {x=nx; y=ny; z=nz;}

	// operator
	inline MQPoint& operator = (const MQPoint& p)
		{x=p.x; y=p.y; z=p.z; return *this;}
	inline MQPoint& operator = (const float& s)
		{x=y=z=s; return *this;}
	inline MQPoint& operator += (const MQPoint& p)
		{x+=p.x; y+=p.y; z+=p.z; return *this;}
	inline MQPoint& operator += (const float& s)
		{x+=s; y+=s; z+=s; return *this;}
	inline MQPoint& operator -= (const MQPoint& p)
		{x-=p.x; y-=p.y; z-=p.z; return *this;}
	inline MQPoint& operator -= (const float& s)
		{x-=s; y-=s; z-=s; return *this;}
	inline MQPoint& operator *= (const MQPoint& p)
		{x*=p.x; y*=p.y; z*=p.z; return *this;}
	inline MQPoint& operator *= (float s)
		{x*=s; y*=s; z*=s; return *this;}
	inline MQPoint& operator /= (float s)
		{x/=s; y/=s; z/=s; return *this;}
	friend inline MQPoint operator + (const MQPoint& p1, const MQPoint& p2)
		{return MQPoint(p1.x+p2.x, p1.y+p2.y, p1.z+p2.z);}
	friend inline MQPoint operator + (const MQPoint& p, const float& s)
		{return MQPoint(p.x+s, p.y+s, p.z+s);}
	friend inline MQPoint operator + (const float& s, const MQPoint& p)
		{return MQPoint(p.x+s, p.y+s, p.z+s);}
	friend inline MQPoint operator - (const MQPoint& p1, const MQPoint& p2)
		{return MQPoint(p1.x-p2.x, p1.y-p2.y, p1.z-p2.z);}
	friend inline MQPoint operator - (const MQPoint& p, const float& s)
		{return MQPoint(p.x-s, p.y-s, p.z-s);}
	friend inline MQPoint operator - (const float& s, const MQPoint& p)
		{return MQPoint(s-p.x, s-p.y, s-p.z);}
	friend inline MQPoint operator * (const MQPoint& p1, const MQPoint& p2)
		{return MQPoint(p1.x*p2.x, p1.y*p2.y, p1.z*p2.z);}
	friend inline MQPoint operator * (const MQPoint& p, const float& s)
		{return MQPoint(p.x*s, p.y*s, p.z*s);}
	friend inline MQPoint operator * (const float& s, const MQPoint& p)
		{return MQPoint(p.x*s, p.y*s, p.z*s);}
	friend inline MQPoint operator / (const MQPoint& p, const float& s)
		{return MQPoint(p.x/s, p.y/s, p.z/s);}
	friend inline bool operator == (const MQPoint& p1, const MQPoint& p2)
		{return (p1.x==p2.x && p1.y==p2.y && p1.z==p2.z);}
	friend inline bool operator != (const MQPoint& p1, const MQPoint& p2)
		{return (p1.x!=p2.x || p1.y!=p2.y || p1.z!=p2.z);}
} MQPoint;

typedef struct MQColor
{
	float r,g,b;
public:
	inline MQColor() {}
	inline MQColor(float value) { r=g=b=value; }
	inline MQColor(float nr, float ng, float nb) { r=nr; g=ng; b=nb; }

	friend inline bool operator == (const MQColor& c1, const MQColor& c2)
		{return (c1.r==c2.r && c1.g==c2.g && c1.b==c2.b);}
	friend inline bool operator != (const MQColor& c1, const MQColor& c2)
		{return (c1.r!=c2.r || c1.g!=c2.g || c1.b!=c2.b);}
} MQColor;

typedef struct MQCoordinate
{
	float u,v;
public:
	inline MQCoordinate() {}
	inline MQCoordinate(float nu, float nv) { u=nu; v=nv; }

	friend inline MQCoordinate operator + (const MQCoordinate& p1, const MQCoordinate& p2)
		{return MQCoordinate(p1.u+p2.u, p1.v+p2.v);}
	friend inline MQCoordinate operator + (const MQCoordinate& p, const float& s)
		{return MQCoordinate(p.u+s, p.v+s);}
	friend inline MQCoordinate operator + (const float& s, const MQCoordinate& p)
		{return MQCoordinate(p.u+s, p.v+s);}
	friend inline MQCoordinate operator - (const MQCoordinate& p1, const MQCoordinate& p2)
		{return MQCoordinate(p1.u-p2.u, p1.v-p2.v);}
	friend inline MQCoordinate operator - (const MQCoordinate& p, const float& s)
		{return MQCoordinate(p.u-s, p.v-s);}
	friend inline MQCoordinate operator - (const float& s, const MQCoordinate& p)
		{return MQCoordinate(s-p.u, s-p.v);}
	friend inline MQCoordinate operator * (const MQCoordinate& p1, const MQCoordinate& p2)
		{return MQCoordinate(p1.u*p2.u, p1.v*p2.v);}
	friend inline MQCoordinate operator * (const MQCoordinate& p, const float& s)
		{return MQCoordinate(p.u*s, p.v*s);}
	friend inline MQCoordinate operator * (const float& s, const MQCoordinate& p)
		{return MQCoordinate(p.u*s, p.v*s);}
	friend inline MQCoordinate operator / (const MQCoordinate& p, const float& s)
		{return MQCoordinate(p.u/s, p.v/s);}
	friend inline MQCoordinate operator / (const MQCoordinate& p1, const MQCoordinate& p2)
		{return MQCoordinate(p1.u/p2.u, p1.v/p2.v);}
	friend inline bool operator == (const MQCoordinate& p1, const MQCoordinate& p2)
		{return (p1.u==p2.u && p1.v==p2.v);}
	friend inline bool operator != (const MQCoordinate& p1, const MQCoordinate& p2)
		{return (p1.u!=p2.u || p1.v!=p2.v);}
} MQCoordinate;

typedef struct {
	DWORD dwSize;
	int type;
	float scale;
	const char *softname;
	int axis_x;
	int axis_y;
	int axis_z;
} MQFileDialogInfo;

typedef struct MQSelectVertex
{
	int object; //index of object
	int vertex; //index of vertex

	inline MQSelectVertex() {}
	inline MQSelectVertex(int i, int j) { object=i; vertex=j; }
	friend inline bool operator == (const MQSelectVertex& l1, const MQSelectVertex& l2)
		{return (l1.object==l2.object && l1.vertex==l2.vertex);}
	friend inline bool operator != (const MQSelectVertex& l1, const MQSelectVertex& l2)
		{return (l1.object!=l2.object || l1.vertex!=l2.vertex);}
} MQSelectVertex;

typedef struct MQSelectLine
{
	int object; //index of object
	int face; //index of face
	int line; //index of line

	inline MQSelectLine() {}
	inline MQSelectLine(int i, int j, int k) { object=i; face=j; line=k; }
	friend inline bool operator == (const MQSelectLine& l1, const MQSelectLine& l2)
		{return (l1.object==l2.object && l1.face==l2.face && l1.line==l2.line);}
	friend inline bool operator != (const MQSelectLine& l1, const MQSelectLine& l2)
		{return (l1.object!=l2.object || l1.face!=l2.face || l1.line!=l2.line);}
} MQSelectLine;

typedef struct MQSelectFace
{
	int object; //index of object
	int face; //face of index

	inline MQSelectFace() {}
	inline MQSelectFace(int i, int j) { object=i; face=j; }
	friend inline bool operator == (const MQSelectFace& l1, const MQSelectFace& l2)
		{return (l1.object==l2.object && l1.face==l2.face);}
	friend inline bool operator != (const MQSelectFace& l1, const MQSelectFace& l2)
		{return (l1.object!=l2.object || l1.face!=l2.face);}
} MQSelectFace;


typedef BYTE MQBool;


#ifdef MQPLUGIN_EXPORTS

class MQCObject;

typedef void *MQDocument;
typedef void *MQObject;
typedef void *MQMaterial;

// global
MQPLUGIN_API HWND MQ_GetWindowHandle(void);
MQPLUGIN_API MQObject MQ_CreateObject(void);
MQPLUGIN_API MQMaterial MQ_CreateMaterial(void);
MQPLUGIN_API void MQ_ShowFileDialog(const char *title, MQFileDialogInfo *info);
MQPLUGIN_API void MQ_ImportAxis(MQFileDialogInfo *info, MQPoint *pts, int pts_count);
MQPLUGIN_API void MQ_ExportAxis(MQFileDialogInfo *info, MQPoint *pts, int pts_count);

// class MQDodument
MQPLUGIN_API int MQDoc_GetObjectCount(MQDocument doc);
MQPLUGIN_API MQObject MQDoc_GetObject(MQDocument doc, int index);
MQPLUGIN_API int MQDoc_GetCurrentObjectIndex(MQDocument doc);
MQPLUGIN_API void MQDoc_SetCurrentObjectIndex(MQDocument doc, int index);
MQPLUGIN_API int MQDoc_AddObject(MQDocument doc, MQObject obj);
MQPLUGIN_API void MQDoc_DeleteObject(MQDocument doc, int index);
MQPLUGIN_API int MQDoc_GetObjectIndex(MQDocument doc, MQObject obj);
MQPLUGIN_API int MQDoc_GetMaterialCount(MQDocument doc);
MQPLUGIN_API MQMaterial MQDoc_GetMaterial(MQDocument doc, int material);
MQPLUGIN_API int MQDoc_GetCurrentMaterialIndex(MQDocument doc);
MQPLUGIN_API void MQDoc_SetCurrentMaterialIndex(MQDocument doc, int index);
MQPLUGIN_API int MQDoc_AddMaterial(MQDocument doc, MQMaterial mat);
MQPLUGIN_API void MQDoc_DeleteMaterial(MQDocument doc, int index);
MQPLUGIN_API void MQDoc_Compact(MQDocument doc);
MQPLUGIN_API void MQDoc_ClearSelect(MQDocument doc, DWORD flag);
MQPLUGIN_API BOOL MQDoc_AddSelectVertex(MQDocument doc, int objindex, int vertindex);
MQPLUGIN_API BOOL MQDoc_DeleteSelectVertex(MQDocument doc, int objindex, int vertindex);
MQPLUGIN_API BOOL MQDoc_IsSelectVertex(MQDocument doc, int objindex, int vertindex);
MQPLUGIN_API BOOL MQDoc_AddSelectFace(MQDocument doc, int objindex, int faceindex);
MQPLUGIN_API BOOL MQDoc_DeleteSelectFace(MQDocument doc, int objindex, int faceindex);
MQPLUGIN_API BOOL MQDoc_IsSelectFace(MQDocument doc, int objindex, int faceindex);


// class MQObject
MQPLUGIN_API void MQObj_Delete(MQObject obj);
MQPLUGIN_API MQObject MQObj_Clone(MQObject obj);
MQPLUGIN_API void MQObj_Merge(MQObject dest, MQObject source);
MQPLUGIN_API void MQObj_Freeze(MQObject obj, DWORD flag);
MQPLUGIN_API void MQObj_GetName(MQObject obj, char *buffer, int size);
MQPLUGIN_API int  MQObj_GetVertexCount(MQObject obj);
MQPLUGIN_API void MQObj_GetVertex(MQObject obj, int index, MQPoint *pts);
MQPLUGIN_API void MQObj_SetVertex(MQObject obj, int index, MQPoint *pts);
MQPLUGIN_API void MQObj_GetVertexArray(MQObject obj, MQPoint *ptsarray);
MQPLUGIN_API int  MQObj_GetFaceCount(MQObject obj);
MQPLUGIN_API int  MQObj_GetFacePointCount(MQObject obj, int face);
MQPLUGIN_API void MQObj_GetFacePointArray(MQObject obj, int face, int *vertex);
MQPLUGIN_API void MQObj_GetFaceCoordinateArray(MQObject obj, int face, MQCoordinate *uvarray);
MQPLUGIN_API int  MQObj_GetFaceMaterial(MQObject obj, int face);
MQPLUGIN_API void MQObj_SetName(MQObject obj, const char *buffer);
MQPLUGIN_API int  MQObj_AddVertex(MQObject obj, MQPoint *p);
MQPLUGIN_API BOOL MQObj_DeleteVertex(MQObject obj, int index, BOOL del_vert);
MQPLUGIN_API int  MQObj_GetVertexRefCount(MQObject obj, int index);
MQPLUGIN_API DWORD MQObj_GetVertexColor(MQObject obj, int index);
MQPLUGIN_API void MQObj_SetVertexColor(MQObject obj, int index, DWORD color);
MQPLUGIN_API float MQObj_GetVertexWeight(MQObject obj, int index);
MQPLUGIN_API void MQObj_SetVertexWeight(MQObject obj, int index, float value);
MQPLUGIN_API void MQObj_CopyVertexAttribute(MQObject obj, int vert1, MQObject obj2, int vert2);
MQPLUGIN_API int  MQObj_AddFace(MQObject obj, int count, int *index);
MQPLUGIN_API BOOL MQObj_DeleteFace(MQObject obj, int index, BOOL del_vert);
MQPLUGIN_API BOOL MQObj_InvertFace(MQObject obj, int index);
MQPLUGIN_API void MQObj_SetFaceMaterial(MQObject obj, int face, int material);
MQPLUGIN_API void MQObj_SetFaceCoordinateArray(MQObject obj, int face, MQCoordinate *uvarray);
MQPLUGIN_API void MQObj_OptimizeVertex(MQObject obj, float distance, MQBool *apply);
MQPLUGIN_API void MQObj_Compact(MQObject obj);
MQPLUGIN_API DWORD MQObj_GetVisible(MQObject obj);
MQPLUGIN_API void MQObj_SetVisible(MQObject obj, DWORD visible);
MQPLUGIN_API DWORD MQObj_GetPatchType(MQObject obj);
MQPLUGIN_API void MQObj_SetPatchType(MQObject obj, DWORD type);
MQPLUGIN_API int  MQObj_GetPatchSegment(MQObject obj);
MQPLUGIN_API void MQObj_SetPatchSegment(MQObject obj, int segment);

// class MQMaterial
MQPLUGIN_API void MQMat_Delete(MQMaterial mat);
MQPLUGIN_API void MQMat_GetName(MQMaterial mat, char *buffer, int size);
MQPLUGIN_API void MQMat_GetColor(MQMaterial mat, MQColor *color);
MQPLUGIN_API float MQMat_GetAlpha(MQMaterial mat);
MQPLUGIN_API float MQMat_GetDiffuse(MQMaterial mat);
MQPLUGIN_API float MQMat_GetAmbient(MQMaterial mat);
MQPLUGIN_API float MQMat_GetEmission(MQMaterial mat);
MQPLUGIN_API float MQMat_GetSpecular(MQMaterial mat);
MQPLUGIN_API float MQMat_GetPower(MQMaterial mat);
MQPLUGIN_API void MQMat_GetTextureName(MQMaterial mat, char *buffer, int size);
MQPLUGIN_API void MQMat_GetAlphaName(MQMaterial mat, char *buffer, int size);
MQPLUGIN_API void MQMat_GetBumpName(MQMaterial mat, char *buffer, int size);
MQPLUGIN_API void MQMat_SetName(MQMaterial mat, const char *name);
MQPLUGIN_API void MQMat_SetColor(MQMaterial mat, MQColor *color);
MQPLUGIN_API void MQMat_SetAlpha(MQMaterial mat, float value);
MQPLUGIN_API void MQMat_SetDiffuse(MQMaterial mat, float value);
MQPLUGIN_API void MQMat_SetAmbient(MQMaterial mat, float value);
MQPLUGIN_API void MQMat_SetEmission(MQMaterial mat, float value);
MQPLUGIN_API void MQMat_SetSpecular(MQMaterial mat, float value);
MQPLUGIN_API void MQMat_SetPower(MQMaterial mat, float value);
MQPLUGIN_API void MQMat_SetTextureName(MQMaterial mat, const char *name);
MQPLUGIN_API void MQMat_SetAlphaName(MQMaterial mat, const char *name);
MQPLUGIN_API void MQMat_SetBumpName(MQMaterial mat, const char *name);


extern DWORD (*MQCheckVersion)(DWORD exe_version);
extern BOOL (*MQInit)(const char *exe_name);
extern const char *(*MQGetPlugInName)(void);
extern void (*MQGetPlugInID)(DWORD *Product, DWORD *ID);
extern int (*MQGetPlugInType)(void);
extern const char *(*MQEnumString)(int index);
extern const char *(*MQEnumFileType)(int index);
extern const char *(*MQEnumFileExt)(int index);
extern BOOL (*MQExportFile)(int index, const char *filename, MQDocument doc);
extern BOOL (*MQImportFile)(int index, const char *filename, MQDocument doc);
extern BOOL (*MQCreate)(int index, MQDocument doc);
extern BOOL (*MQModifyObject)(int index, MQDocument doc);

#else //MQPLUGIN_EXPORTS

class MQCDocument;
class MQCObject;
class MQCMaterial;

typedef MQCDocument *MQDocument;
typedef MQCObject *MQObject;
typedef MQCMaterial *MQMaterial;


// global
extern HWND (__stdcall *MQ_GetWindowHandle)(void);
extern MQObject (__stdcall *MQ_CreateObject)(void);
extern MQMaterial (__stdcall *MQ_CreateMaterial)(void);
extern void (__stdcall *MQ_ShowFileDialog)(const char *title, MQFileDialogInfo *info);
extern void (__stdcall *MQ_ImportAxis)(MQFileDialogInfo *info, MQPoint *pts, int pts_count);
extern void (__stdcall *MQ_ExportAxis)(MQFileDialogInfo *info, MQPoint *pts, int pts_count);

// class MQDodument
extern int  (__stdcall *MQDoc_GetObjectCount)(MQDocument doc);
extern MQObject (__stdcall *MQDoc_GetObject)(MQDocument doc, int index);
extern int  (__stdcall *MQDoc_GetCurrentObjectIndex)(MQDocument doc);
extern void (__stdcall *MQDoc_SetCurrentObjectIndex)(MQDocument doc, int index);
extern int  (__stdcall *MQDoc_AddObject)(MQDocument doc, MQObject obj);
extern void (__stdcall *MQDoc_DeleteObject)(MQDocument doc, int index);
extern int  (__stdcall *MQDoc_GetObjectIndex)(MQDocument doc, MQObject obj);
extern int  (__stdcall *MQDoc_GetMaterialCount)(MQDocument doc);
extern MQMaterial (__stdcall *MQDoc_GetMaterial)(MQDocument doc, int material);
extern int  (__stdcall *MQDoc_GetCurrentMaterialIndex)(MQDocument doc);
extern void (__stdcall *MQDoc_SetCurrentMaterialIndex)(MQDocument doc, int index);
extern int  (__stdcall *MQDoc_AddMaterial)(MQDocument doc, MQMaterial mat);
extern void (__stdcall *MQDoc_DeleteMaterial)(MQDocument doc, int index);
extern void (__stdcall *MQDoc_Compact)(MQDocument doc);
extern void (__stdcall *MQDoc_ClearSelect)(MQDocument doc, DWORD flag);
extern BOOL (__stdcall *MQDoc_AddSelectVertex)(MQDocument doc, int objindex, int vertindex);
extern BOOL (__stdcall *MQDoc_DeleteSelectVertex)(MQDocument doc, int objindex, int vertindex);
extern BOOL (__stdcall *MQDoc_IsSelectVertex)(MQDocument doc, int objindex, int vertindex);
extern BOOL (__stdcall *MQDoc_AddSelectLine)(MQDocument doc, int objindex, int faceindex, int lineindex);
extern BOOL (__stdcall *MQDoc_DeleteSelectLine)(MQDocument doc, int objindex, int faceindex, int lineindex);
extern BOOL (__stdcall *MQDoc_IsSelectLine)(MQDocument doc, int objindex, int faceindex, int lineindex);
extern BOOL (__stdcall *MQDoc_AddSelectFace)(MQDocument doc, int objindex, int faceindex);
extern BOOL (__stdcall *MQDoc_DeleteSelectFace)(MQDocument doc, int objindex, int faceindex);
extern BOOL (__stdcall *MQDoc_IsSelectFace)(MQDocument doc, int objindex, int faceindex);

// class MQObject
extern void (__stdcall *MQObj_Delete)(MQObject obj);
extern MQObject (__stdcall *MQObj_Clone)(MQObject obj);
extern void (__stdcall *MQObj_Merge)(MQObject dest, MQObject source);
extern void (__stdcall *MQObj_Freeze)(MQObject obj, DWORD flag);
extern void (__stdcall *MQObj_GetName)(MQObject obj, char *buffer, int size);
extern int  (__stdcall *MQObj_GetVertexCount)(MQObject obj);
extern void (__stdcall *MQObj_GetVertex)(MQObject obj, int index, MQPoint *pts);
extern void (__stdcall *MQObj_SetVertex)(MQObject obj, int index, MQPoint *pts);
extern void (__stdcall *MQObj_GetVertexArray)(MQObject obj, MQPoint *ptsarray);
extern int  (__stdcall *MQObj_GetFaceCount)(MQObject obj);
extern int  (__stdcall *MQObj_GetFacePointCount)(MQObject obj, int face);
extern void (__stdcall *MQObj_GetFacePointArray)(MQObject obj, int face, int *vertex);
extern void (__stdcall *MQObj_GetFaceCoordinateArray)(MQObject obj, int face, MQCoordinate *uvarray);
extern int  (__stdcall *MQObj_GetFaceMaterial)(MQObject obj, int face);
extern void (__stdcall *MQObj_SetName)(MQObject obj, const char *buffer);
extern int  (__stdcall *MQObj_AddVertex)(MQObject obj, MQPoint *p);
extern BOOL (__stdcall *MQObj_DeleteVertex)(MQObject obj, int index, BOOL del_vert);
extern int  (__stdcall *MQObj_GetVertexRefCount)(MQObject obj, int index);
extern DWORD (__stdcall *MQObj_GetVertexColor)(MQObject obj, int index);
extern void (__stdcall *MQObj_SetVertexColor)(MQObject obj, int index, DWORD color);
extern float (__stdcall *MQObj_GetVertexWeight)(MQObject obj, int index);
extern void (__stdcall *MQObj_SetVertexWeight)(MQObject obj, int index, float value);
extern void (__stdcall *MQObj_CopyVertexAttribute)(MQObject obj, int vert1, MQObject obj2, int vert2);
extern int  (__stdcall *MQObj_AddFace)(MQObject obj, int count, int *index);
extern BOOL (__stdcall *MQObj_DeleteFace)(MQObject obj, int index, BOOL del_vert);
extern BOOL (__stdcall *MQObj_InvertFace)(MQObject obj, int index);
extern void (__stdcall *MQObj_SetFaceMaterial)(MQObject obj, int face, int material);
extern void (__stdcall *MQObj_SetFaceCoordinateArray)(MQObject obj, int face, MQCoordinate *uvarray);
extern void (__stdcall *MQObj_OptimizeVertex)(MQObject obj, float distance, MQBool *apply);
extern void (__stdcall *MQObj_Compact)(MQObject obj);
extern DWORD (__stdcall *MQObj_GetVisible)(MQObject obj);
extern void (__stdcall *MQObj_SetVisible)(MQObject obj, DWORD visible);
extern DWORD (__stdcall *MQObj_GetPatchType)(MQObject obj);
extern void (__stdcall *MQObj_SetPatchType)(MQObject obj, DWORD type);
extern int  (__stdcall *MQObj_GetPatchSegment)(MQObject obj);
extern void (__stdcall *MQObj_SetPatchSegment)(MQObject obj, int segment);

// class MMaterial
extern void (__stdcall *MQMat_Delete)(MQMaterial mat);
extern void (__stdcall *MQMat_GetName)(MQMaterial mat, char *buffer, int size);
extern void (__stdcall *MQMat_GetColor)(MQMaterial mat, MQColor *color);
extern float (__stdcall *MQMat_GetAlpha)(MQMaterial mat);
extern float (__stdcall *MQMat_GetDiffuse)(MQMaterial mat);
extern float (__stdcall *MQMat_GetAmbient)(MQMaterial mat);
extern float (__stdcall *MQMat_GetEmission)(MQMaterial mat);
extern float (__stdcall *MQMat_GetSpecular)(MQMaterial mat);
extern float (__stdcall *MQMat_GetPower)(MQMaterial mat);
extern void (__stdcall *MQMat_GetTextureName)(MQMaterial mat, char *buffer, int size);
extern void (__stdcall *MQMat_GetAlphaName)(MQMaterial mat, char *buffer, int size);
extern void (__stdcall *MQMat_GetBumpName)(MQMaterial mat, char *buffer, int size);
extern void (__stdcall *MQMat_SetName)(MQMaterial mat, const char *name);
extern void (__stdcall *MQMat_SetColor)(MQMaterial mat, MQColor *color);
extern void (__stdcall *MQMat_SetAlpha)(MQMaterial mat, float value);
extern void (__stdcall *MQMat_SetDiffuse)(MQMaterial mat, float value);
extern void (__stdcall *MQMat_SetAmbient)(MQMaterial mat, float value);
extern void (__stdcall *MQMat_SetEmission)(MQMaterial mat, float value);
extern void (__stdcall *MQMat_SetSpecular)(MQMaterial mat, float value);
extern void (__stdcall *MQMat_SetPower)(MQMaterial mat, float value);
extern void (__stdcall *MQMat_SetTextureName)(MQMaterial mat, const char *name);
extern void (__stdcall *MQMat_SetAlphaName)(MQMaterial mat, const char *name);
extern void (__stdcall *MQMat_SetBumpName)(MQMaterial mat, const char *name);


class MQCDocument
{
public:
	inline int GetObjectCount(void) { return MQDoc_GetObjectCount(this); }
	inline MQObject GetObject(int index) { return MQDoc_GetObject(this, index); }
	inline int GetCurrentObjectIndex(void) { return MQDoc_GetCurrentObjectIndex(this); }
	inline void SetCurrentObjectIndex(int index) { MQDoc_SetCurrentObjectIndex(this, index); }
	inline int AddObject(MQObject obj) { return MQDoc_AddObject(this, obj); }
	inline void DeleteObject(int index) { MQDoc_DeleteObject(this, index); }
	inline int GetObjectIndex(MQObject obj) { return MQDoc_GetObjectIndex(this, obj); }
	inline int GetMaterialCount(void) { return MQDoc_GetMaterialCount(this); }
	inline MQMaterial GetMaterial(int material) { return MQDoc_GetMaterial(this, material); }
	inline int GetCurrentMaterialIndex(void) { return MQDoc_GetCurrentMaterialIndex(this); }
	inline void SetCurrentMaterialIndex(int index) { MQDoc_SetCurrentMaterialIndex(this, index); }
	inline int AddMaterial(MQMaterial mat) { return MQDoc_AddMaterial(this, mat); }
	inline void DeleteMaterial(int index) { MQDoc_DeleteMaterial(this, index); }
	inline void Compact(void) { MQDoc_Compact(this); }
	inline void ClearSelect(DWORD flag) { MQDoc_ClearSelect(this, flag); }
	inline BOOL AddSelectVertex(int objindex, int vertindex) { return MQDoc_AddSelectVertex(this, objindex, vertindex); }
	inline BOOL DeleteSelectVertex(int objindex, int vertindex) { return MQDoc_DeleteSelectVertex(this, objindex, vertindex); }
	inline BOOL IsSelectVertex(int objindex, int vertindex) { return MQDoc_IsSelectVertex(this, objindex, vertindex); }
	inline BOOL AddSelectVertex(MQSelectVertex sel) { return MQDoc_AddSelectVertex(this, sel.object, sel.vertex); }
	inline BOOL DeleteSelectVertex(MQSelectVertex sel) { return MQDoc_DeleteSelectVertex(this, sel.object, sel.vertex); }
	inline BOOL IsSelectVertex(MQSelectVertex sel) { return MQDoc_IsSelectVertex(this, sel.object, sel.vertex); }
	inline BOOL AddSelectLine(int objindex, int faceindex, int lineindex)
		{ return MQDoc_AddSelectLine(this, objindex, faceindex, lineindex); }
	inline BOOL DeleteSelectLine(int objindex, int faceindex, int lineindex)
		{ return MQDoc_DeleteSelectLine(this, objindex, faceindex, lineindex); }
	inline BOOL IsSelectLine(int objindex, int faceindex, int lineindex)
		{ return MQDoc_IsSelectLine(this, objindex, faceindex, lineindex); }
	inline BOOL AddSelectLine(MQSelectLine sel) { return MQDoc_AddSelectLine(this, sel.object, sel.face, sel.line); }
	inline BOOL DeleteSelectLine(MQSelectLine sel) { return MQDoc_DeleteSelectLine(this, sel.object, sel.face, sel.line); }
	inline BOOL IsSelectLine(MQSelectLine sel) { return MQDoc_IsSelectLine(this, sel.object, sel.face, sel.line); }
	inline BOOL AddSelectFace(int objindex, int faceindex) { return MQDoc_AddSelectFace(this, objindex, faceindex); }
	inline BOOL DeleteSelectFace(int objindex, int faceindex) { return MQDoc_DeleteSelectFace(this, objindex, faceindex); }
	inline BOOL IsSelectFace(int objindex, int faceindex) { return MQDoc_IsSelectFace(this, objindex, faceindex); }
	inline BOOL AddSelectFace(MQSelectFace sel) { return MQDoc_AddSelectFace(this, sel.object, sel.face); }
	inline BOOL DeleteSelectFace(MQSelectFace sel) { return MQDoc_DeleteSelectFace(this, sel.object, sel.face); }
	inline BOOL IsSelectFace(MQSelectFace sel) { return MQDoc_IsSelectFace(this, sel.object, sel.face); }
};

class MQCObject
{
public:
	inline void DeleteThis(void) { MQObj_Delete(this); }
	inline MQObject Clone(void) { return MQObj_Clone(this); }
	inline void Merge(MQObject source) { MQObj_Merge(this, source); }
	inline void Freeze(DWORD flag) { MQObj_Freeze(this, flag); }
	inline void GetName(char *buffer, int size) { MQObj_GetName(this, buffer, size); }
	inline int  GetVertexCount(void) { return MQObj_GetVertexCount(this); }
	inline MQPoint GetVertex(int index) { MQPoint pts; MQObj_GetVertex(this, index, &pts); return pts; }
	inline void SetVertex(int index, MQPoint pts) { MQObj_SetVertex(this, index, &pts); }
	inline void GetVertexArray(MQPoint *ptsarray) { MQObj_GetVertexArray(this, ptsarray); }
	inline int  GetFaceCount(void) { return MQObj_GetFaceCount(this); }
	inline int  GetFacePointCount(int face) { return MQObj_GetFacePointCount(this, face); }
	inline void GetFacePointArray(int face, int *vertex) { MQObj_GetFacePointArray(this, face, vertex); }
	inline void GetFaceCoordinateArray(int face, MQCoordinate *uvarray) { MQObj_GetFaceCoordinateArray(this, face, uvarray); }
	inline int  GetFaceMaterial(int face) { return MQObj_GetFaceMaterial(this, face); }
	inline void SetName(const char *buffer) { MQObj_SetName(this, buffer); }
	inline int  AddVertex(MQPoint p) { return MQObj_AddVertex(this, &p); }
	inline BOOL DeleteVertex(int index) { return MQObj_DeleteVertex(this, index, TRUE); }
	inline int  GetVertexRefCount(int index) { return MQObj_GetVertexRefCount(this, index); }
	inline DWORD GetVertexColor(int index) { return MQObj_GetVertexColor(this, index); }
	inline void SetVertexColor(int index, DWORD color) { MQObj_SetVertexColor(this, index, color); }
	inline float GetVertexWeight(int index) { return MQObj_GetVertexWeight(this, index); }
	inline void SetVertexWeight(int index, float value) { MQObj_SetVertexWeight(this, index, value); }
	inline void CopyVertexAttribute(int vert1, MQObject obj2, int vert2) { MQObj_CopyVertexAttribute(this, vert1, obj2, vert2); }
	inline int  AddFace(int count, int *index) { return MQObj_AddFace(this, count, index); }
	inline BOOL DeleteFace(int index) { return MQObj_DeleteFace(this, index, TRUE); }
	inline BOOL InvertFace(int index) { return MQObj_InvertFace(this, index); }
	inline void SetFaceMaterial(int face, int material) { MQObj_SetFaceMaterial(this, face, material); }
	inline void SetFaceCoordinateArray(int face, MQCoordinate *uvarray) { MQObj_SetFaceCoordinateArray(this, face, uvarray); }
	inline void OptimizeVertex(float distance, MQBool *apply) { MQObj_OptimizeVertex(this, distance, apply); }
	inline void Compact(void) { MQObj_Compact(this); }
	inline DWORD GetVisible(void) { return MQObj_GetVisible(this); }
	inline void SetVisible(DWORD visible) { MQObj_SetVisible(this, visible); }
	inline DWORD GetPatchType(void) { return MQObj_GetPatchType(this); }
	inline void SetPatchType(DWORD type) { MQObj_SetPatchType(this, type); }
	inline int  GetPatchSegment(void) { return MQObj_GetPatchSegment(this); }
	inline void SetPatchSegment(int segment) { MQObj_SetPatchSegment(this, segment); }
};

class MQCMaterial
{
public:
	inline void DeleteThis(void) { MQMat_Delete(this); }
	inline void GetName(char *buffer, int size) { MQMat_GetName(this, buffer, size); }
	inline MQColor GetColor(void) { MQColor color; MQMat_GetColor(this, &color); return color; }
	inline float GetAlpha(void) { return MQMat_GetAlpha(this); }
	inline float GetDiffuse(void) { return MQMat_GetDiffuse(this); }
	inline float GetAmbient(void) { return MQMat_GetAmbient(this); }
	inline float GetEmission(void) { return MQMat_GetEmission(this); }
	inline float GetSpecular(void) { return MQMat_GetSpecular(this); }
	inline float GetPower(void) { return MQMat_GetPower(this); }
	inline void GetTextureName(char *buffer, int size) { MQMat_GetTextureName(this, buffer, size); }
	inline void GetAlphaName(char *buffer, int size) { MQMat_GetAlphaName(this, buffer, size); }
	inline void GetBumpName(char *buffer, int size) { MQMat_GetBumpName(this, buffer, size); }
	inline void SetName(const char *name) { MQMat_SetName(this, name); }
	inline void SetColor(MQColor color) { MQMat_SetColor(this, &color); }
	inline void SetAlpha(float value) { MQMat_SetAlpha(this, value); }
	inline void SetDiffuse(float value) { MQMat_SetDiffuse(this, value); }
	inline void SetAmbient(float value) { MQMat_SetAmbient(this, value); }
	inline void SetEmission(float value) { MQMat_SetEmission(this, value); }
	inline void SetSpecular(float value) { MQMat_SetSpecular(this, value); }
	inline void SetPower(float value) { MQMat_SetPower(this, value); }
	inline void SetTextureName(const char *name) { MQMat_SetTextureName(this, name); }
	inline void SetAlphaName(const char *name) { MQMat_SetAlphaName(this, name); }
	inline void SetBumpName(const char *name) { MQMat_SetBumpName(this, name); }
};


#endif //MQPLUGIN_EXPORTS

#endif //MQPluginH


